# Automarker v2.0
#  Management of individual student submissions and deadlines
# hussein suleman
# 12 march 2010

package Manage;

use XML::DOM;
use DateTime::Format::HTTP;
use DateTime::Duration;
use DateTime;

use User;

sub new 
{
   my ($classname, $cgi, $config, $vula, $userObj) = @_;
   
   my $self = { 
      cgi           => $cgi,
      config        => $config,
      vula          => $vula,
      userObj       => $userObj,
   };
   bless $self, $classname;
   
   return $self;
}


sub updateExtras
{
   my ($self) = @_;
   
   if ($self->{'userObj'}->checkAdmin == 0) { return; }
   
   my $managedUserObj = new User ($self->{'cgi'}, $self->{'config'}, $self->{'cgi'}->param ('manageduser'), $self->{'cgi'}->param ('managedrole'));
   my $assignmentList = $managedUserObj->readSummary;
   
   my $added = 0;
   foreach my $assignment (@$assignmentList)
   {
      if ($assignment->[0] eq $self->{'cgi'}->param ('id'))
      {
         $assignment->[1] = $self->{'cgi'}->param ('extratries');
         $assignment->[2] = $self->{'cgi'}->param ('extradays');
         $added = 1;
      }
   }
   if ($added == 0)
   {
      push (@$assignmentList, [ $self->{'cgi'}->param ('id'), 
              $self->{'cgi'}->param ('extratries'), $self->{'cgi'}->param ('extradays'), [] ]);
   }
   
   $managedUserObj->writeSummary ($assignmentList);
   
   $self->assignmentTable ();
}

sub assignmentTable
{
   my ($self) = @_;
   
   my $openAssignments = '';
   my $closedAssignments = '';
   my $switchLink = '';
   
   my $managedUserObj;
   if ($self->{'cgi'}->param ('manageduser') ne '')
   {
      $managedUserObj = new User ($self->{'cgi'}, $self->{'config'}, $self->{'cgi'}->param ('manageduser'), $self->{'cgi'}->param ('managedrole'));

      if ($self->{'userObj'}->checkAdmin)
      {
         my $environment = $ENV{QUERY_STRING};
         $environment =~ s/proxyuser\=[\@\.\_a-zA-Z0-9\+]+//go;
         $environment =~ s/proxyrole\=[\@\.\_a-zA-Z0-9\+]+//go;
         $environment =~ s/manageduser\=[\@\.\_a-zA-Z0-9\+]+//go;
         $environment =~ s/managedrole\=[\@\.\_a-zA-Z0-9\+]+//go;
         $environment =~ s/action\=[\@\.\_a-zA-Z0-9\+]+//go;
         $environment =~ s/\&+/\&/go;
         $id = $self->{'cgi'}->param ('manageduser');
         $role = $self->{'cgi'}->param ('managedrole');
         $role =~ s/ /\+/go;
         $switchLink = " : <a href=\"?$environment&proxyuser=$id&proxyrole=$role\">$managedUserObj->{'realUser'}</a>";
      }   
   }
   else
   {
      $managedUserObj = $self->{'userObj'};
   }

   my @vulaAssignmentList = $self->{'vula'}->getAssignmentList;
   my $assignmentList = $managedUserObj->readSummary;

   foreach my $vulaAssignment (@vulaAssignmentList)
   {
      my $title = $vulaAssignment->{'title'};
      if ($title =~ /Feedback/)
      { next; }

      my $dueTime = $vulaAssignment->{'dueTime'};
      my $closeTime = $vulaAssignment->{'closeTime'};
      my $openTime = $vulaAssignment->{'openTime'};

      my $tries = $self->{'config'}->{'maximumSubmissions'};
      if (-e $self->{'config'}->{'assignmentsdir'}.'/'.$title.'/markingguide.xml')
      {
         my $parser = new XML::DOM::Parser;
         my $markingGuide = $parser->parsefile ($self->{'config'}->{'assignmentsdir'}.'/'.$title.'/markingguide.xml');
      
         my $triesNode = $markingGuide->getElementsByTagName ('tries');
         if (($triesNode->getLength == 1) &&
             ($triesNode->item(0)->hasChildNodes) &&
             ($triesNode->item(0)->getFirstChild->toString ne ''))
         { $tries = $triesNode->item(0)->getFirstChild->toString; }
      }

      my $submissions = '';
      my $numberOfTries = 0;
      my $extraTries = 0;
      my $extraDays = 0;
      foreach my $sassignment (@$assignmentList)
      {
         if ($title eq $sassignment->[0])
         {
            my $environment = $ENV{QUERY_STRING};
            $environment =~ s/action\=[\@\.\_a-zA-Z0-9\+]+//go;
            $environment =~ s/id\=[\@\.\_a-zA-Z0-9\+]+//go;
            $environment =~ s/markee\=[\@\.\_a-zA-Z0-9\+]+//go;
            $environment =~ s/filename\=[\@\.\_a-zA-Z0-9\+]+//go;
            $environment =~ s/attemptnumber\=[\@\.\_a-zA-Z0-9\+]+//go;
            $environment =~ s/\&+/\&/go;
            
            foreach my $attempt (@{$sassignment->[3]})
            {
               $numberOfTries++;
               my $score = $attempt->[1];
               my $filename = $attempt->[0];
               my $timestring = localtime ((stat ($self->{'config'}->{'submissionsdir'}.'/'.$managedUserObj->{'effectiveUser'}."/$title/$numberOfTries/$filename"))[9]);

               $submissions .=  $self->{'userObj'}->toXML ($filename)."</a>&nbsp;[$score]&nbsp;$timestring";
               if (-e $self->{'config'}->{'submissionsdir'}.'/'.$managedUserObj->{'effectiveUser'}."/$title/$numberOfTries/marking_report.txt")
               {
                  $submissions .= "&nbsp;[<a href=\"?$environment&action=viewreport&markee=".$managedUserObj->{'effectiveUser'}."&id=$title&attemptnumber=$numberOfTries&filename=marking_report.txt\">report</a>]";
               }
               $submissions .= "&nbsp;[<a href=\"?$environment&action=download&markee=".$managedUserObj->{'effectiveUser'}."&id=$title&attemptnumber=$numberOfTries&filename=$filename\">download</a>]";
               $submissions .= "&nbsp;[<a target=\"sourcecode\" href=\"?$environment&action=explode&markee=".$managedUserObj->{'effectiveUser'}."&id=$title&attemptnumber=$numberOfTries&filename=$filename\">explode</a>]";
               $submissions .= '<br/>';
            }
            
            # add in manual marking results if they exist
            if (-e $self->{'config'}->{'markingdir'}.'/'.$title.'/'.$managedUserObj->{'effectiveUser'}.'.xml')
            {
               my $parser = new XML::DOM::Parser;
               my $template = $parser->parsefile ($self->{'config'}->{'markingdir'}.'/'.$title.'/'.$managedUserObj->{'effectiveUser'}.'.xml');
               my $score = '';
               foreach my $scoreNode ($template->getElementsByTagName ('total'))
               {
                  $score = $scoreNode->getFirstChild->toString;
               }
               $submissions .= "Manual marking&nbsp;[$score]&nbsp;[<a href=\"?$environment&action=viewfeedback&markee=".$managedUserObj->{'effectiveUser'}."&id=$title\">feedback</a>]<br/>";
            }
            
            $extraTries = $sassignment->[1];
            $extraDays = $sassignment->[2];
         }
      }
      $submissions = '<p>'.$submissions.'</p>';
      
      my $dateColour = '#000000';
      my $triesColour = '#000000';
      my $extraTime = DateTime::Duration->new ( days => $extraDays );
      my $nowTime = DateTime->now ( time_zone => $self->{'config'}->{'timezone'} );
      
      if (
          ($self->{'cgi'}->param ('manageduser') eq '')
          &&
          (
           (
            ((($closeTime + $extraTime) - $nowTime)->is_positive) 
            && 
            (($nowTime - $openTime)->is_positive) 
            &&
            ($numberOfTries < ($tries+$extraTries))
           ) 
           ||
           ($self->{'userObj'}->checkMarker)
          )
         )
      {
         $submissions .= "<p><form enctype=\"multipart/form-data\" method=\"post\">".
                         "<input type=\"hidden\" name=\"action\" value=\"submit\"/>".
                         "<input type=\"hidden\" name=\"user\" value=\"".$self->{'cgi'}->param ('user')."\"/>".
                         "<input type=\"hidden\" name=\"proxyuser\" value=\"".$self->{'cgi'}->param ('proxyuser')."\"/>".
                         "<input type=\"hidden\" name=\"proxyrole\" value=\"".$self->{'cgi'}->param ('proxyrole')."\"/>".
                         "<input type=\"file\" name=\"submission\"/>".
                         "<input type=\"hidden\" name=\"id\" value=\"$title\"/>".
                         "<input type=\"hidden\" name=\"site\" value=\"".$self->{'cgi'}->param ('site')."\"/>".
                         "<input type=\"hidden\" name=\"role\" value=\"".$self->{'cgi'}->param ('role')."\"/>".
                         "<input type=\"submit\" value=\"Submit\"/></form></p>";
      }
      else
      {
         $submissions .= '&nbsp;';
      }
      
      my $penalty = '';
      if (($nowTime - ($closeTime + $extraTime))->is_positive)
      { $dateColour = '#ff0000'; }
      elsif (($nowTime - ($dueTime + $extraTime))->is_positive)
      { 
         $dateColour = '#0000ff';
         my $almostOneDay = DateTime::Duration->new ( hours=>23, minutes=>59, seconds=>59);
         my $lateness = ($nowTime + $almostOneDay) - ($dueTime + $extraTime);
         my $daysLate = $lateness->days + ($lateness->weeks * 7);
         $penalty = $self->{'config'}->{'penaltyperday'} * $daysLate;
         if ($penalty > 100)
         { $penalty = 100; }
         $penalty = '<p>Penalty (if you submit now): '.$penalty.'%</p>';
      }
      if ($numberOfTries >= ($tries+$extraTries))
      { $triesColour = '#ff0000'; }
      
      my $triesForm = '';
      my $dateForm = '';
      
      if (($self->{'cgi'}->param ('manageduser') ne '') &&
          ($self->{'userObj'}->checkAdmin))
      {
         $dateForm = "<form enctype=\"multipart/form-data\" method=\"post\">".
                         "<input type=\"hidden\" name=\"action\" value=\"updateextras\"/>".
                         "<input type=\"hidden\" name=\"user\" value=\"".$self->{'cgi'}->param ('user')."\"/>".
                         "<input type=\"hidden\" name=\"proxyuser\" value=\"".$self->{'cgi'}->param ('proxyuser')."\"/>".
                         "<input type=\"hidden\" name=\"proxyrole\" value=\"".$self->{'cgi'}->param ('proxyrole')."\"/>".
                         "<input type=\"hidden\" name=\"id\" value=\"$title\"/>".
                         "<input type=\"hidden\" name=\"site\" value=\"".$self->{'cgi'}->param ('site')."\"/>".
                         "<input type=\"hidden\" name=\"role\" value=\"".$self->{'cgi'}->param ('role')."\"/>".
                         "<input type=\"hidden\" name=\"manageduser\" value=\"".$self->{'cgi'}->param ('manageduser')."\"/>".
                         "<p>Extra days: ".
                         "<input size=\"5\" type=\"text\" name=\"extradays\" value=\"$extraDays\"/>".
                         "<input type=\"submit\" value=\"Update\"/>".
                         "</p>";
         $triesForm = "<p>Extra tries: ".
                         "<input size=\"5\" type=\"text\" name=\"extratries\" value=\"$extraTries\"/>".
                         "<input type=\"submit\" value=\"Update\"/>".
                         "</p></form>";
      }
      
      my $dueTimeAdjusted = '';
      my $triesAdjusted = '';
      if ($extraDays > 0)
      { $dueTimeAdjusted = '<p>Adjusted Due Date: '.($dueTime + $extraTime).'</p>'; }
      if ($extraTries > 0)
      { $triesAdjusted = "<p>Adjusted Attempts: $numberoftries / ".($tries + $extraTries).'</p>'; }
      
      $title =~ s/ /&nbsp;/go;
      my $assignmentEntry = 
                 "<tr><td>$title</td>".
                 "<td style=\"color: $dateColour\">$dueTime$dateForm$dueTimeAdjusted$penalty</td>".
                 "<td style=\"color: $triesColour\">$numberOfTries / $tries$triesForm$triesAdjusted</td>".
                 "<td>$submissions</td></tr>";
      if (
            ((($closeTime + $extraTime) - $nowTime)->is_positive) 
            && 
            (($nowTime - $openTime)->is_positive) 
            &&
            ($numberOfTries < ($tries+$extraTries))
         ) 
      { $openAssignments .= $assignmentEntry; }
      else
      { $closedAssignments .= $assignmentEntry; }           
   }

   my $output = '';   
   $output .= "<h3>Open Assignments$switchLink</h3>".
              '<table>'.
              "<tr><th>Assignment</th><th>Due Date</th><th>Attempts</th><th width=\"60%\">Submissions</th></tr>".
              $openAssignments.
              '</table>';
   $output .= "<h3>Closed Assignments$switchLink</h3>".
              '<table>'.
              "<tr><th>Assignment</th><th>Due Date</th><th>Attempts</th><th width=\"60%\">Submissions</th></tr>".
              $closedAssignments.
              '</table>';
   print $output;
}

sub manageList
{
   my ($self) = @_;
   
   my $output = '<h3>List of Users to Manage</h3>'.
                '<table><tr><th>Name</th><th>ID</th></tr>';
   
   my @vulaSiteMembers = $self->{'vula'}->getSiteMembers;
   
   foreach my $vulaStudent (@vulaSiteMembers)
   {
      my $firstName = $vulaStudent->{'firstName'};
      my $lastName = $vulaStudent->{'lastName'};
      my $id = $vulaStudent->{'id'};
      my $role = $vulaStudent->{'role'};
      
      if ($id ne '')
      {
         my $environment = $ENV{QUERY_STRING};
         $environment =~ s/manageduser\=[\@\.\_a-zA-Z0-9\+]+//go;
         $environment =~ s/managedrole\=[\@\.\_a-zA-Z0-9\+]+//go;
         $environment =~ s/action\=[\@\.\_a-zA-Z0-9\+]+//go;
         $environment =~ s/\&+/\&/go;
         $role =~ s/ /\+/go;
         $output .= "<tr><td>$firstName $lastName</td>".
                    "<td><a href=\"?$environment&manageduser=$id&managedrole=$role&action=manage\">$id</a></td></tr>";
      }
   }
   
   $output .= '</table>';
   print $output;
}

1;
