# Automarker v2.0
#  Manual marking by tutors
# hussein suleman
# 12 march 2010

package Marking;

use XML::DOM;

use LockB;
use User;

sub new 
{
   my ($classname, $cgi, $config, $vula, $userObj) = @_;
   
   my $self = { 
      cgi           => $cgi,
      config        => $config,
      vula          => $vula,
      userObj       => $userObj,
   };
   bless $self, $classname;
   
   return $self;
}

sub getSubfield
{
   my ($self, $node, $field, $default) = @_;
   
   my $valuenode = $node->getElementsByTagName ($field);
   if ($valuenode->getLength > 0)
   {
      $valuenode = $valuenode->item(0);
      if ($valuenode->hasChildNodes)
      {
         return $valuenode->getFirstChild->toString;
      }
   }
   return $default;
}

sub toXML 
{
   my ($self, $data) = @_;
   $data =~ s/\&/&amp;/go;
   $data =~ s/\</&lt;/go;
   $data =~ s/\>/&gt;/go;
   $data =~ s/'/&apos;/go;
   $data =~ s/"/&quot;/go;
   $data;
}

sub fromXML 
{
   my ($self, $data) = @_;
   $data =~ s/\&lt;/\</go;
   $data =~ s/\&gt;/\>/go;
   $data =~ s/\&apos;/'/go;
   $data =~ s/\&quot;/"/go;
   $data =~ s/\&amp;/\&/go;
   $data;
}

sub getAssignmentList
{
   my ($self) = @_;
   
   my @assignmentlist = ();
   
   my @vulaAssignmentList = $self->{'vula'}->getAssignmentList;

   foreach my $vulaAssignment (@vulaAssignmentList)
   {
      my $title = $vulaAssignment->{'title'};
      mkdir $self->{'config'}->{'markingdir'}.'/'.$title;

      if ((-e $self->{'config'}->{'assignmentsdir'}.'/'.$title.'/markingtemplate.xml') &&
          ((-s $self->{'config'}->{'assignmentsdir'}.'/'.$title.'/markingtemplate.xml')>0))
      {
         push (@assignmentlist, { title         => $title, 
                                  id            => $vulaAssignment->{'id'},
                                });
      }
   }
   
   @assignmentlist;
}

sub getMarkee
{
   my ($self, $title) = @_;
   
   my @vulaSiteMembers = $self->{'vula'}->getSiteMembers;
   
   srand;
   
   my $lock = new LockB ($self->{'config'}->{'markingdir'}."/$title/lockfile");

   while ($#vulaSiteMembers > -1)
   {
      my $position = int (rand ($#vulaSiteMembers+1));
      my $id = $vulaSiteMembers[$position]->{'id'};
      my $role = $vulaSiteMembers[$position]->{'role'};

      my $markeeObj = new User ($self->{'cgi'}, $self->{'config'}, $id, 'Student');
      my @bestAttempt = $markeeObj->getBestAttempt ($title);
      
      if (($#bestAttempt == 2) &&
          ($role eq 'Student') &&
          (! -e $self->{'config'}->{'markingdir'}."/$title/$id.xml") &&
          ((! -e $self->{'config'}->{'markingdir'}."/$title/$id.lock") || 
          (((-M $self->{'config'}->{'markingdir'}."/$title/$id.lock")*24*60) > 0.01)))
      {
         system ("touch \"".$self->{'config'}->{'markingdir'}."/$title/$id.lock\"");
         $lock->unlock;
         return $id;
      }
      else
      { splice (@vulaSiteMembers, $position, 1); }
   }
   
   $lock->unlock;
   return undef;
}

sub printMarkeeList
{
   my ($self, $title) = @_;
   
   my $environment = $ENV{QUERY_STRING};
   $environment =~ s/id\=[\@\.\_a-zA-Z0-9\+]+//go;
   $environment =~ s/\&+/\&/go;

   my @vulaSiteMembers = $self->{'vula'}->getSiteMembers;
   
   my $output = '';
   
   for ( my $i=0; $i<=$#vulaSiteMembers; $i++ )
   {
      my $firstName = $vulaSiteMembers[$i]->{'firstName'};
      my $lastName = $vulaSiteMembers[$i]->{'lastName'};
      my $id = $vulaSiteMembers[$i]->{'id'};
      my $role = $vulaSiteMembers[$i]->{'role'};

      my $markeeObj = new User ($self->{'cgi'}, $self->{'config'}, $id, 'Student');
      my @bestAttempt = $markeeObj->getBestAttempt ($title);
      
      if (($#bestAttempt == 2) &&
          ($role eq 'Student') &&
          (! -e $self->{'config'}->{'markingdir'}."/$title/$id.xml"))
      {
         $output .= "<tr><td>$firstName $lastName</td><td><a href=\"?$environment&id=$title&markee=$id\">$id</a></td></tr>";
      }
   }
   
   if ($output eq '')
   { return undef; }
   
   print "<h3>Marking: $title</h3><table><tr><th>Name</th><th>ID</th></tr>".$output."</table>";
   return 1;
}

sub printTemplate
{
   my ($self, $title, $markee) = @_;
   
   if (! -e $self->{'config'}->{'assignmentsdir'}.'/'.$title.'/markingtemplate.xml')
   { return; }
   
   my $parser = new XML::DOM::Parser;
   my $template = $parser->parsefile ($self->{'config'}->{'assignmentsdir'}.'/'.$title.'/markingtemplate.xml');
   
   print "<h3>Marking: $markee</h3>";
   
   # find attempt number and filename
   my $markeeObj = new User ($self->{'cgi'}, $self->{'config'}, $markee, 'Student');
   my @bestAttempt = $markeeObj->getBestAttempt ($title);

   if ($#bestAttempt == 2)
   {
      my $environment = $ENV{QUERY_STRING};
      $environment =~ s/action\=[\@\.\_a-zA-Z0-9\+]+//go;
      $environment =~ s/id\=[\@\.\_a-zA-Z0-9\+]+//go;
      $environment =~ s/filename\=[\@\.\_a-zA-Z0-9\+]+//go;
      $environment =~ s/attemptnumber\=[\@\.\_a-zA-Z0-9\+]+//go;
      $environment =~ s/manageduser\=[\@\.\_a-zA-Z0-9\+]+//go;
      $environment =~ s/\&+/\&/go;
      print "<p>Explode and view <a target=\"sourcecode\" href=\"?$environment&action=explode&id=$title&attemptnumber=".$bestAttempt[0]."&filename=".$bestAttempt[1]."&manageduser=$markee\">".
            $markeeObj->toXML ($bestAttempt[1])."</a>&nbsp;[automark:".$bestAttempt[2]."]";
      if (-e $self->{'config'}->{'submissionsdir'}.'/'.$markee.'/'.$title.'/'.$bestAttempt[0].'/marking_report.txt')
      {
         print "&nbsp;[<a href=\"?$environment&action=viewreport&markee=$markee&id=$title&attemptnumber=$bestAttempt[0]&filename=marking_report.txt\">".
               "Report</a>]";
      }
      print "&nbsp;[<a href=\"?$environment&action=download&markee=$markee&id=$title&attemptnumber=$bestAttempt[0]&filename=".$bestAttempt[1]."\">".
               "Download</a>]";
      print "</p>";
   }

   print "<p><form method=\"post\">".
         "<input type=\"hidden\" name=\"action\" value=\"marking\"/>".
         "<input type=\"hidden\" name=\"user\" value=\"".$self->{'cgi'}->param ('user')."\"/>".
         "<input type=\"hidden\" name=\"proxyuser\" value=\"".$self->{'cgi'}->param ('proxyuser')."\"/>".
         "<input type=\"hidden\" name=\"proxyrole\" value=\"".$self->{'cgi'}->param ('proxyrole')."\"/>".
         "<input type=\"hidden\" name=\"markedassignment\" value=\"$title\"/>".
         "<input type=\"hidden\" name=\"marker\" value=\"".$self->{'userObj'}->{'effectiveUser'}."\"/>".
         "<input type=\"hidden\" name=\"markee\" value=\"$markee\"/>".
         "<input type=\"hidden\" name=\"site\" value=\"".$self->{'cgi'}->param ('site')."\"/>".
         "<input type=\"hidden\" name=\"role\" value=\"".$self->{'cgi'}->param ('role')."\"/>".
         "<table><tr><th>Description</th><th>Marks</th><th>Max</th></tr>";
   
   my $generalcomments = '';
   foreach my $field ($template->getElementsByTagName ('generalcomments'))
   {
      $generalcomments = $field->getFirstChild->toString;
   }

   my $fieldnumber = 'field1';
   foreach my $field ($template->getElementsByTagName ('field'))
   {
      my $description = '';
      foreach my $descNode ($field->getElementsByTagName ('description'))
      {
         $description = $descNode->getFirstChild->toString;
      }
      my $marks = 0;
      foreach my $marksNode ($field->getElementsByTagName ('marks'))
      {
         $marks = $marksNode->getFirstChild->toString;
      }

      my $input = '';
      for ( my $i=0; $i<=$marks; $i++ )
      {
         $input .= "<option value=\"$i\">$i</option>";
      }
      print "<tr><td>$description</td>".
            "<td><select name=\"$fieldnumber\"/>$input</select></td>".
            "<td>$marks</td></tr>";

      $fieldnumber++;
   }
   
   print "<tr><th>General Comments</th>".
         "<td colspan=\"2\"><textarea name=\"fieldcomments\" rows=\"5\" cols=\"40\">".$generalcomments."</textarea></td></tr>".
         "<tr><td colspan=\"3\" style=\"textalign:center;\">".
         "<input type=\"submit\" value=\"Submit\"/>".
         "</td></tr>";
}

sub saveFeedback
{
   my ($self) = @_;
   
   my $marker = $self->{'cgi'}->param ('marker');
   my $markedassignment = $self->{'cgi'}->param ('markedassignment');
   my $markee = $self->{'cgi'}->param ('markee');
   my $generalcomments = $self->{'cgi'}->param ('fieldcomments');
   
   my $parser = new XML::DOM::Parser;
   my $template = $parser->parsefile ($self->{'config'}->{'assignmentsdir'}.'/'.$markedassignment.'/markingtemplate.xml');

   my $datastore = "<feedback>\n".
                   "<markedassignment>$markedassignment</markedassignment>\n".
                   "<marker>$marker</marker>\n".
                   "<markee>$markee</markee>\n";
                      
   my $feedback = "Feedback on $markedassignment\nSubmitter: $markee\nMarker: $marker\n\n";
   my $fieldnumber = 'field1';
   my $total = 0;
   foreach my $field ($template->getElementsByTagName ('field'))
   {
      my $description = '';
      foreach my $descNode ($field->getElementsByTagName ('description'))
      {
         $description = $descNode->getFirstChild->toString;
      }
      my $maxMarks = 0;
      foreach my $marksNode ($field->getElementsByTagName ('marks'))
      {
         $maxMarks = $marksNode->getFirstChild->toString;
      }
      my $marks += $self->{'cgi'}->param ($fieldnumber);
      $total += $marks;
      $feedback .= "$description [$marks / $maxMarks]\n";
      $datastore .= "<field><description>$description</description><marks>$marks</marks><maxMarks>$maxMarks</maxMarks></field>\n";
      $fieldnumber++;
   }
   $feedback .= "\nTotal : [$total]\n";
   $datastore .= "<total>$total</total>\n";
   $feedback .= "\nGeneral Comments\n".$self->toXML($generalcomments)."\n";
   $datastore .= "<generalComments>".$self->toXML($generalcomments)."</generalComments>\n</feedback>\n";

   my @assignmentList = $self->getAssignmentList;
   
   my $assignmentId = '';
   for ( my $j=0; $j<=$#assignmentList; $j++ )
   {
      if ($assignmentList[$j]->{'title'} eq $markedassignment)
      { $assignmentId = $assignmentList[$j]->{'id'}; }
   }
   
   open (my $outfile, '>'.$self->{'config'}->{'markingdir'}.'/'.$markedassignment.'/'.$markee.'.xml');
   print $outfile $datastore;
   close ($outfile);

   my $userObj = new User ($self->{'cgi'}, $self->{'config'}, $markee, 'Student');
   $feedback =~ s/\n/\<br\/\>/go;
   $self->{'vula'}->setMarkAndComments ($assignmentId, $userObj, $total, $feedback);

   my $lock = new LockB ($self->{'config'}->{'markingdir'}."/$title/lockfile");
   unlink ($self->{'config'}->{'markingdir'}."/$title/$markee.lock");
   $lock->unlock;
}

sub viewFeedback
{
   my ($self) = @_;
   
   my $markedassignment = $self->{'cgi'}->param ('id');
   my $markee = $self->{'cgi'}->param ('markee');
   
   my $parser = new XML::DOM::Parser;
   my $template = $parser->parsefile ($self->{'config'}->{'markingdir'}.'/'.$markedassignment.'/'.$markee.'.xml');

   my $marker = '';
   foreach my $markerNode ($template->getElementsByTagName ('marker'))
   { $marker = $markerNode->getFirstChild->toString; }
   my $total = 0;
   foreach my $totalNode ($template->getElementsByTagName ('total'))
   { $total = $totalNode->getFirstChild->toString; }
   my $generalComments = '';
   foreach my $gcNode ($template->getElementsByTagName ('generalComments'))
   { $generalComments = $gcNode->getFirstChild->toString; }

   my $feedback = "Feedback on $markedassignment\nSubmitter: $markee\nMarker: $marker\n\n";

   foreach my $field ($template->getElementsByTagName ('field'))
   {
      my $description = '';
      foreach my $descNode ($field->getElementsByTagName ('description'))
      { $description = $descNode->getFirstChild->toString; }
      my $maxMarks = 0;
      foreach my $marksNode ($field->getElementsByTagName ('maxMarks'))
      { $maxMarks = $marksNode->getFirstChild->toString; }
      my $marks = 0;
      foreach my $marksNode ($field->getElementsByTagName ('marks'))
      { $marks = $marksNode->getFirstChild->toString; }
      $feedback .= "$description [$marks / $maxMarks]\n";
   }
   $feedback .= "\nTotal : [$total]\n";
   $feedback .= "\nGeneral Comments\n".$generalComments."\n";
   $feedback =~ s/\n/\<br\/\>/go;
   
   print $feedback;
}

sub reviewMarkingList
{
   my ($self, $title) = @_;
   
   print "<h3>Review Marking List</h3>";
   
   # calculate total number of submissions
   my $numberOfSubmissions = 0;
   opendir (my $dir, $self->{'config'}->{'submissionsdir'});
   while (my $markee = readdir ($dir))
   {
      if ($markee !~ /^\.+$/)
      {
         my $markeeObj = new User ($self->{'cgi'}, $self->{'config'}, $markee, 'Student');
         my @bestAttempt = $markeeObj->getBestAttempt ($title);
         if ($#bestAttempt == 2) 
         { $numberOfSubmissions++; }
      }
   }
   closedir ($dir);   
         
   my %markers = ();
   my $markedSubmissions = 0;
   
   opendir (my $dir, $self->{'config'}->{'markingdir'}."/$title");
   while (my $filename = readdir ($dir))
   {
      if ($filename =~ /\.xml$/) 
      {
         my $markee = substr ($filename, 0, -4);
         
         my $parser = new XML::DOM::Parser;
         my $feedbackDocument = $parser->parsefile ($self->{'config'}->{'markingdir'}."/$title/$filename");
         my $marker = '';
         foreach my $markerNode ($feedbackDocument->getElementsByTagName ('marker'))
         {
            $marker = $markerNode->getFirstChild->toString;
         }
         my $total = 0;
         foreach my $totalNode ($feedbackDocument->getElementsByTagName ('total'))
         {
            $total = $totalNode->getFirstChild->toString;
         }
         
         if (! exists $markers{$marker})
         { $markers{$marker} = []; }
         
         push (@{$markers{$marker}}, [ $markee, $total ] );
         
         $markedSubmissions++;
      }
   }
   closedir ($dir);
   
   my $environment = $ENV{QUERY_STRING};
   $environment =~ s/markee\=[\@\.\_a-zA-Z0-9\+]+//go;
   $environment =~ s/\&+/\&/go;
   
   print "<p>Number of marked submissions : $markedSubmissions / $numberOfSubmissions</p>";

   print "<table><tr>";
   foreach my $marker (keys %markers)
   {
      if (($marker eq $self->{'userObj'}->{'effectiveUser'}) ||
          ($self->{'userObj'}->checkAdmin))
      {
         print "<td><table>".
               "<tr><th colspan=\"2\">Marker: $marker (".($#{@{$markers{$marker}}}+1).")</th></tr>".
               "<tr><th>Student</th><th>Mark</th></tr>";
         foreach my $markeeNode (@{$markers{$marker}})
         {
            my $markee = $markeeNode->[0];
            my $mark = $markeeNode->[1];
            print "<tr><td><a href=\"?$environment&markee=$markee\">$markee</a></td><td>$mark</td></tr>";
         }
         print "</table></td>";
      }    
   }
   print "</tr></table>";
}

sub reviewMarkingEntry
{
   my ($self, $title, $markee) = @_;
   
   my $parser = new XML::DOM::Parser;
   my $template = $parser->parsefile ($self->{'config'}->{'markingdir'}.'/'.$title."/$markee.xml");
   
   print "<h3>Reviewing Marking: $markee</h3>";
   
   # find attempt number and filename
   my $markeeObj = new User ($self->{'cgi'}, $self->{'config'}, $markee, 'Student');
   my @bestAttempt = $markeeObj->getBestAttempt ($title);

   if ($#bestAttempt == 2)
   {
      my $environment = $ENV{QUERY_STRING};
      $environment =~ s/action\=[\@\.\_a-zA-Z0-9\+]+//go;
      $environment =~ s/id\=[\@\.\_a-zA-Z0-9\+]+//go;
      $environment =~ s/filename\=[\@\.\_a-zA-Z0-9\+]+//go;
      $environment =~ s/attemptnumber\=[\@\.\_a-zA-Z0-9\+]+//go;
      $environment =~ s/manageduser\=[\@\.\_a-zA-Z0-9\+]+//go;
      $environment =~ s/\&+/\&/go;
      print "<p>Explode and view <a target=\"sourcecode\" href=\"?$environment&action=explode&id=$title&attemptnumber=".$bestAttempt[0]."&filename=".$bestAttempt[1]."&manageduser=$markee\">".
            $markeeObj->toXML ($bestAttempt[1])."</a>&nbsp;[automark:".$bestAttempt[2]."]";
      if (-e $self->{'config'}->{'submissionsdir'}.'/'.$markee.'/'.$title.'/'.$bestAttempt[0].'/marking_report.txt')
      {
         print "&nbsp;[<a href=\"?$environment&action=viewreport&markee=$markee&id=$title&attemptnumber=$bestAttempt[0]&filename=marking_report.txt\">".
               "Report</a>]";
      }
      print "&nbsp;[<a href=\"?$environment&action=download&markee=$markee&id=$title&attemptnumber=$bestAttempt[0]&filename=".$bestAttempt[1]."\">".
               "Download</a>]";
      print "</p>";
   }

   print "<p><form method=\"post\">".
         "<input type=\"hidden\" name=\"action\" value=\"reviewmarking\"/>".
         "<input type=\"hidden\" name=\"user\" value=\"".$self->{'cgi'}->param ('user')."\"/>".
         "<input type=\"hidden\" name=\"proxyuser\" value=\"".$self->{'cgi'}->param ('proxyuser')."\"/>".
         "<input type=\"hidden\" name=\"proxyrole\" value=\"".$self->{'cgi'}->param ('proxyrole')."\"/>".
         "<input type=\"hidden\" name=\"markedassignment\" value=\"$title\"/>".
         "<input type=\"hidden\" name=\"marker\" value=\"".$self->{'userObj'}->{'effectiveUser'}."\"/>".
         "<input type=\"hidden\" name=\"markee\" value=\"$markee\"/>".
         "<input type=\"hidden\" name=\"site\" value=\"".$self->{'cgi'}->param ('site')."\"/>".
         "<input type=\"hidden\" name=\"role\" value=\"".$self->{'cgi'}->param ('role')."\"/>".
         "<table><tr><th>Description</th><th>Marks</th><th>Max</th></tr>";
   
   my $fieldnumber = 'field1';
   foreach my $field ($template->getElementsByTagName ('field'))
   {
      my $description = '';
      foreach my $descNode ($field->getElementsByTagName ('description'))
      {
         $description = $descNode->getFirstChild->toString;
      }
      my $marks = 0;
      foreach my $marksNode ($field->getElementsByTagName ('marks'))
      {
         $marks = $marksNode->getFirstChild->toString;
      }
      my $maxMarks = 0;
      foreach my $maxMarksNode ($field->getElementsByTagName ('maxMarks'))
      {
         $maxMarks = $maxMarksNode->getFirstChild->toString;
      }

      my $input = '';
      for ( my $i=0; $i<=$maxMarks; $i++ )
      {
         my $selected = '';
         if ($i == $marks)
         { $selected = ' selected'; }
         $input .= "<option value=\"$i\"$selected>$i</option>";
      }
      print "<tr><td>$description</td>".
            "<td><select name=\"$fieldnumber\"/>$input</select></td>".
            "<td>$maxMarks</td></tr>";

      $fieldnumber++;
   }
   
   my $generalComments = '';
   foreach my $generalCommentsNode ($template->getElementsByTagName ('generalComments'))
   {
      if ($generalCommentsNode->hasChildNodes)
      {
         $generalComments = $generalCommentsNode->getFirstChild->toString;
      }
   }
   print "<tr><th>General Comments</th>".
         "<td colspan=\"2\"><textarea name=\"fieldcomments\" rows=\"5\" cols=\"40\">".$generalComments."</textarea></td></tr>".
         "<tr><td colspan=\"3\" style=\"textalign:center;\">".
         "<input type=\"submit\" value=\"Submit\"/>".
         "</td></tr>";
}

sub reviewmarking
{
   my ($self) = @_;

   if ($self->{'userObj'}->checkMarker == 0) { return; }
   
   if ($self->{'cgi'}->param ('markedassignment') eq '')
   {
      my $output = '<h3>List of Reviewable Assignments</h3>'.
                   '<table><tr><th>Name</th></tr>';
                
      my @markingassignmentlist = $self->getAssignmentList;

      foreach my $assignment (@markingassignmentlist)
      {
         my $environment = $ENV{QUERY_STRING};
         $environment =~ s/markedassignment\=[\@\.\_a-zA-Z0-9\+]+//go;
         $environment =~ s/\&+/\&/go;
         $output .= "<tr><td><a href=\"?$environment&markedassignment=".
                    $assignment->{'title'}."\">".$assignment->{'title'}.
                    "</a></td></tr>";
      }
      
      $output .= '</table>';
      print $output;
   }
   else
   {
      my $title = $self->{'cgi'}->param ('markedassignment');
      my $feedback = 0;
      if ($self->{'cgi'}->param ('marker') ne '')
      {
         $self->saveFeedback;
         $feedback = 1;
      }
      if (($self->{'cgi'}->param ('markee') eq '') || ($feedback == 1))
      {
         $self->reviewMarkingList ($title);
      }
      else
      {
         $self->reviewMarkingEntry ($title, $self->{'cgi'}->param ('markee'));
      }
   }
} 

sub marking
{
   my ($self) = @_;
   
   if ($self->{'userObj'}->checkMarker == 0) { return; }
   
   if ($self->{'cgi'}->param ('markedassignment') eq '')
   {
      my $output = '<h3>List of Markable Assignments</h3>'.
                   '<table><tr><th>Name</th></tr>';
                
      my @markingassignmentlist = $self->getAssignmentList;

      foreach my $assignment (@markingassignmentlist)
      {
         my $environment = $ENV{QUERY_STRING};
         $environment =~ s/markedassignment\=[\@\.\_a-zA-Z0-9\+]+//go;
         $environment =~ s/\&+/\&/go;
         $output .= "<tr><td><a href=\"?$environment&markedassignment=".
                    $assignment->{'title'}."\">".$assignment->{'title'}.
                    "</a></td></tr>";
      }
      
      $output .= '</table>';
      print $output;
   }
   else
   {
      my $title = $self->{'cgi'}->param ('markedassignment');
      if ($self->{'cgi'}->param ('marker') ne '')
      {
         $self->saveFeedback;
      }

      if (! -e $self->{'config'}->{'assignmentsdir'}.'/'.$title.'/markingtemplate.xml')
      { return; }
   
      my $parser = new XML::DOM::Parser;
      my $template = $parser->parsefile ($self->{'config'}->{'assignmentsdir'}.'/'.$title.'/markingtemplate.xml');
   
      if ($template->getElementsByTagName ('nonRandom')->getLength() > 0)
      {
         my $markee = $self->{'cgi'}->param ('markee');
         if (($markee ne '') && ($self->{'cgi'}->param ('marker') eq ''))
         {
            $self->printTemplate ($title, $markee);
         }
         else
         {   
            my $markee = $self->printMarkeeList ($title);
            if (! defined $markee)
            {
               print "Finished marking!";
            }
         }
      }
      else
      {
         my $markee = $self->getMarkee ($title);
         if (! defined $markee)
         {
            print "Finished marking!";
         }
         else
         {
            $self->printTemplate ($title, $markee);
         }
      }   
   }
   
} 

1;
