# Automarker v2.0
#  User management module
# hussein suleman
# 12 march 2010

package User;

use XML::DOM;

sub new 
{
   my ($classname, $cgi, $config, $user, $role) = @_;
   
   my $self = { 
      realUser      => $user,
      effectiveUser => $user,
      realRole      => $role,
      effectiveRole => $role,
      cgi           => $cgi,
      config        => $config,
   };
   bless $self, $classname;
   
   if (($self->checkAdmin == 1) && ($self->{'cgi'}->param ('proxyuser') ne '') && 
       ($self->{'cgi'}->param ('proxyrole') ne ''))
   {
      $self->{'effectiveUser'} = $self->{'cgi'}->param ('proxyuser');
      $self->{'effectiveRole'} = $self->{'cgi'}->param ('proxyrole');
   }
   
   return $self;
}

sub checkRealAdmin
{
   my ($self) = @_;
   
   my $role = $self->{'realRole'};
   foreach my $adminrole (@{$self->{'config'}->{'adminroles'}})
   {
      if ($role eq $adminrole)
      { return 1; }
   }
   return 0;
}

sub checkAdmin
{
   my ($self) = @_;
   
   my $role = $self->{'effectiveRole'};
   foreach my $adminrole (@{$self->{'config'}->{'adminroles'}})
   {
      if ($role eq $adminrole)
      { return 1; }
   }
   return 0;
}

sub checkMarker
{
   my ($self) = @_;
   
   my $role = $self->{'effectiveRole'};
   foreach my $markerrole (@{$self->{'config'}->{'markerroles'}})
   {
      if ($role eq $markerrole)
      { return 1; }
   }
   return 0;
}

sub getSubfield
{
   my ($self, $node, $field, $default) = @_;
   
   my $valuenode = $node->getElementsByTagName ($field);
   if ($valuenode->getLength > 0)
   {
      $valuenode = $valuenode->item(0);
      if ($valuenode->hasChildNodes)
      {
         return $valuenode->getFirstChild->toString;
      }
   }
   return $default;
}

sub toXML 
{
   my ($self, $data) = @_;
   $data =~ s/\&/&amp;/go;
   $data =~ s/\</&lt;/go;
   $data =~ s/\>/&gt;/go;
   $data =~ s/'/&apos;/go;
   $data =~ s/"/&quot;/go;
   $data;
}

sub fromXML 
{
   my ($self, $data) = @_;
   $data =~ s/\&lt;/\</go;
   $data =~ s/\&gt;/\>/go;
   $data =~ s/\&apos;/'/go;
   $data =~ s/\&quot;/"/go;
   $data =~ s/\&amp;/\&/go;
   $data;
}

sub readSummary
{
   my ($self) = @_;
   
   my $assignmentlist = [];
   my $user = $self->{'effectiveUser'};
   if (-e $self->{'config'}->{'submissionsdir'}."/$user/summary.xml")
   {
      my $parser = new XML::DOM::Parser;
      my $assignments = $parser->parsefile ($self->{'config'}->{'submissionsdir'}."/$user/summary.xml");
      if (defined $assignments)
      {
         foreach my $assignment ($assignments->getElementsByTagName ("assignment"))
         {
            my $assignmententry = [];
            my $id = $self->getSubfield ($assignment, 'id', '');
            my $extratries = $self->getSubfield ($assignment, 'extratries', 0);
            my $extradays = $self->getSubfield ($assignment, 'extradays', 0);
            foreach my $attempt ($assignment->getElementsByTagName ("attempt"))
            {
               my $filename = $self->fromXML ($self->getSubfield ($attempt, 'filename', ''));
               my $score = $self->getSubfield ($attempt, 'score', 0);
               push (@$assignmententry, [$filename, $score]);
            }
            push (@$assignmentlist, [$id, $extratries, $extradays, $assignmententry]);
         }
      }
   }
   $assignmentlist;
}

sub getBestAttempt
{
   my ($self, $title) = @_;
   
   my $assignmentlist = $self->readSummary;

   if ($#$assignmentlist == -1)
   { return (); }
   
   foreach my $assignment (@$assignmentlist)
   {
      if ($assignment->[0] eq $title)
      {
         if ($#{$assignment->[3]} == -1)
         { return (); }

         my $bestScore = -1;
         my $bestFilename = '';
         my $bestAttempt = -1;
         my $anAttempt = 1;
         foreach my $pair (@{$assignment->[3]})
         {
            while ($pair->[1] > 100) { $pair->[1] /= 10; }
            if ($pair->[1] >= $bestScore)
            {
               $bestScore = $pair->[1];
               $bestFilename = $pair->[0]; 
               $bestAttempt = $anAttempt;
            }
            $anAttempt++;
         }
         if ($bestScore > -1)
         {
            return ($bestAttempt, $bestFilename, $bestScore);
         }
      }
   }

   return ();
}

sub writeSummary
{
   my ($self, $assignmentlist) = @_;
   
   my $user = $self->{'effectiveUser'};
   mkdir $self->{'config'}->{'submissionsdir'}."/$user", 0700;

   open ($out, ">".$self->{'config'}->{'submissionsdir'}."/$user/summary.xml");
   print $out "<assignments>\n";
   foreach my $assignment (@$assignmentlist)
   {
      print $out "   <assignment>\n";
      print $out "      <id>$assignment->[0]</id>\n";
      print $out "      <extratries>$assignment->[1]</extratries>\n";
      print $out "      <extradays>$assignment->[2]</extradays>\n";
      foreach my $attempt (@{$assignment->[3]})
      {
         print $out "      <attempt>\n"; 
         print $out "         <filename>".$self->toXML ($attempt->[0])."</filename>\n";
         print $out "         <score>$attempt->[1]</score>\n";
         print $out "      </attempt>\n"; 
      }
      print $out "   </assignment>\n";
   }
   print $out "</assignments>\n";
   close ($out);
}

1;
