# Automarker v2.0
#  Interface with Sakai
# hussein suleman
# 12 march 2010

package Vula;

use DateTime::Format::HTTP;
use SOAP::Lite;
use XML::DOM;

use Config;

sub new 
{
   my ($classname, $cgi, $config) = @_;
   
   my $self = {
      config => $config,
      site   => $cgi->param ('site'),
      url    => $config->{'sakaiurl'}.'/sakai-axis',
      sign   => $cgi->param ('sign'),
      user   => $cgi->param ('user'),
   };
   bless $self, $classname;
   
   $ENV{'HTTPS_CA_FILE'} = $config->{'certfile'};
   $self->getSignedSessionId;
   
   return $self;
}

sub getSignedSessionId
{
   my ($self) = @_;
   
   my $response = SOAP::Lite
     -> service ($self->{'url'}.'/SakaiSigning.jws?wsdl')
     -> getSessionToServer ($self->{'config'}->{'signedobj'});
  
   ($self->{'signedsessionid'}, $self->{'signedurl'}) = split (',', $response);
   $self->{'signedurl'} .= '/sakai-axis';
}

sub getSessionId
{
   my ($self) = @_;
   
   my $response = SOAP::Lite
     -> service ($self->{'url'}.'/SakaiSigning.jws?wsdl')
     -> getsession ($ENV{QUERY_STRING});
  
   $self->{'sessionid'} = $response;
   open (my $f, ">vulaass"); print $f $response.'user='.$self->{'user'}.'&sign='.$self->{'sign'}; close ($f);
}

sub getAssignmentList
{
   my ($self) = @_;
   
   if (!defined $self->{'signedsessionid'}) { return (); }

   my $vulaassignments = undef;
   my $vulaassignmentresult = '';
   while ($vulaassignmentresult eq '')
   {
      $vulaassignmentresult = SOAP::Lite
        -> service ($self->{'signedurl'}.'/WSAssignment.jws?wsdl' )
        -> getAssignmentsForContext ($self->{'signedsessionid'}, $self->{'site'});
   }

   my $parser = new XML::DOM::Parser;
   my @results = ();
      
   if ($vulaassignmentresult =~ /\<Assignments>/)
   {
      my $vulaassignments = $parser->parse ($vulaassignmentresult); 
      
      my @vulaassignmentlist = sort { $self->cleandt ($a->getAttribute ('dueTime')) cmp $self->cleandt ($b->getAttribute ('dueTime')) }
                                      @{$vulaassignments->getElementsByTagName ('Assignment')};
   
      foreach my $vulaassignment (@vulaassignmentlist)
      {
         my $title = $vulaassignment->getAttribute ('title');
         my $id = $vulaassignment->getAttribute ('id');
         my $dueTime = $self->cleandt ($vulaassignment->getAttribute ('dueTime'));
         my $closeTime = $self->cleandt ($vulaassignment->getAttribute ('closeTime'));
         my $openTime = $self->cleandt ($vulaassignment->getAttribute ('openTime'));

         push (@results, { title     => $title,
                           id        => $id,
                           dueTime   => $dueTime,
                           closeTime => $closeTime,
                           openTime  => $openTime } ); 
      }
   }
   
   @results;
}

sub setMarkAndComments
{
   my ($self, $assignmentId, $userObj, $score, $comment) = @_;
   
   my $result = '';
   while ($result ne 'success')
   {
      $result = SOAP::Lite
        -> service ($self->{'signedurl'}.'/WSAssignment.jws?wsdl' )
        -> setAssignmentGradeCommentforUser ($self->{'signedsessionid'}, $assignmentId, $userObj->{'effectiveUser'}, $comment, $score*10);
   }
}

sub cleandt
{
   my ($self, $datestr) = @_;
   DateTime::Format::HTTP->parse_datetime (substr ($datestr, 0, 19));
}

sub getSiteMembers
{
   my ($self) = @_;
   
   if (!defined $self->{'signedsessionid'}) { return (); }
   my @results = ();

   my $vulastudents = undef;
   my $vulastudentresult = '';
   while (($vulastudentresult eq '') || ($vulastudentresult eq 'Failure'))
   {
      $vulastudentresult = SOAP::Lite
       -> service ($self->{'url'}.'/WSAssignment.jws?wsdl')
       -> getSiteMembers ($self->{'signedsessionid'}, $self->{'site'});
   }    
     
   my $parser = new XML::DOM::Parser;
   my $vulastudents = $parser->parse ($vulastudentresult);
     
   my @vulastudentlist = sort { $a->getAttribute ('LastName') cmp $b->getAttribute ('LastName') } $vulastudents->getElementsByTagName ('Members');
   
   foreach my $vulastudent (@vulastudentlist)
   {
      my $firstName = $vulastudent->getAttribute ('FirstName');
      my $lastName = $vulastudent->getAttribute ('LastName');
      my $id = $vulastudent->getAttribute ('id');
      my $role = $vulastudent->getAttribute ('role');
      
      push (@results, { firstName => $firstName,
                        lastName => $lastName,
                        id => $id,
                        role => $role } );
   }
   
   @results;
}

sub checkSakai
{
   my ($self) = @_;
   
   SOAP::Lite
     -> service ($self->{'url'}.'/SakaiSigning.jws?wsdl')
     -> testsign ($ENV{QUERY_STRING});   
}

1;


