#!/usr/bin/perl
#
# parameters: <path_to_file> <tmp_directory> <path_to_assignment>
#

# Automarker v2.0
#  Scoring of assignments
# hussein suleman
# 12 march 2010


use XML::DOM;
use File::Copy;

$|=1;

my $safe = '/home/automark2/safe';
my $diff = '/usr/bin/diff';
my $unzip = '/usr/local/bin/unzip';
my $chmod = '/bin/chmod';
my $nice = '/usr/bin/nice -n 20';

my $maximumCPUDefault = 30;
my $maximumMemoryDefault = 10000000;
my $maximumFileDefault = 1000000;

sub handle_node 
{
   my ($inTrial, $node, $compile, $run, $check1, $check2, $checkScript, $printCheck, $maximumCPU, $maximumMemory, $maximumFile) = @_;
   my $score = 0;
   my $totalScore = 0;
   my $trialNumber = 1;
   
   foreach my $childNode ($node->getChildNodes)
   {
      if ($childNode->getNodeType == ELEMENT_NODE)
      {
         my $nodeName = $childNode->getNodeName;
         if ($nodeName eq 'compile')
         {
            if ($childNode->hasChildNodes)
            {
               $compile = $childNode->getFirstChild->toString;
               if ($compile ne '')
               {
                  $compile = $safe." $maximumCPU $maximumMemory $maximumFile ".$compile;
                  $compile =~ s/&gt;/>/go;
                  $compile =~ s/&lt;/</go;
                  $compile =~ s/&apos;/'/go;
                  $compile =~ s/&quot;/"/go;
                  $compile =~ s/&amp;/&/go;
                  print "\nCompiling and preparing files\n\n";
                  system "cd $ARGV[1]; $compile";
               }
            }
         }
         elsif ($nodeName eq 'run')
         {
            if ($childNode->hasChildNodes)
            {
               $run = $childNode->getFirstChild->toString;
               if ($run ne '')
               {
                  $run = $run;
                  $run =~ s/&gt;/>/go;
                  $run =~ s/&lt;/</go;
                  $run =~ s/&apos;/'/go;
                  $run =~ s/&quot;/"/go;
                  $run =~ s/&amp;/&/go;
               }
            }
         }
         elsif ($nodeName eq 'maximumCPU')
         {
            if ($childNode->hasChildNodes)
            {
               my $newValue = $childNode->getFirstChild->toString;
               if ($newValue =~ /^[0-9]+$/)
               { $maximumCPU = $newValue; }
            }
         }
         elsif ($nodeName eq 'maximumMemory')
         {
            if ($childNode->hasChildNodes)
            {
               my $newValue = $childNode->getFirstChild->toString;
               if ($newValue =~ /^[0-9]+$/)
               { $maximumMemory = $newValue; }
            }
         }
         elsif ($nodeName eq 'maximumFile')
         {
            if ($childNode->hasChildNodes)
            {
               my $newValue = $childNode->getFirstChild->toString;
               if ($newValue =~ /^[0-9]+$/)
               { $maximumFile = $newValue; }
            }
         }
         elsif ($nodeName eq 'file')
         {
            my $filename = $childNode->getAttribute ('name');
            if ($childNode->hasChildNodes)
            {
               my $file = $childNode->getFirstChild->toString;
               if ($file ne '')
               {
                  my @lines = split (/\n/, $file);

                  while (($#lines > -1) && ($lines[$#lines] =~ /^[\n\r\t\s]*$/)) { pop @lines; }
                  while (($#lines > -1) && ($lines[0] =~ /^[\n\r\t\s]*$/)) { shift @lines; }

                  open (my $ifile, ">$ARGV[1]/$filename");
                  foreach my $aline (@lines)
                  {
                     $aline =~ s/&quot;/"/go;
                     $aline =~ s/&apos;/'/go;
                     $aline =~ s/&gt;/\>/go;
                     $aline =~ s/&lt;/\</go;
                     $aline =~ s/&amp;/\&/go;
                     print $ifile $aline."\n";
                  }
                  close ($ifile);
                  system ("chmod a+rwx $ARGV[1]/$filename");
               }
            }
         }
         elsif ($nodeName eq 'fixedfile')
         {
            my $filename = $childNode->getAttribute ('name');
            my $targetFilename =  $childNode->getAttribute ('targetfilename');
            if ($targetFilename eq '')
            {
               $targetFilename = $filename;
            }
            if (($filename ne '') && ($targetFilename ne ''))
            {
               copy ("$ARGV[2]/$filename", "$ARGV[1]/$targetFilename");
               system ("chmod a+rwx $ARGV[1]/$targetFilename");
            }
         }
         elsif ($nodeName eq 'check')
         {
            my $filename1 = $childNode->getElementsByTagName ('file1');
            if (($filename1->getLength == 1) && ($filename1->item(0)->hasChildNodes))
            {
               $check1 = $filename1->item(0)->getFirstChild->toString;
            }
            my $filename2 = $childNode->getElementsByTagName ('file2');
            if (($filename2->getLength == 1) && ($filename2->item(0)->hasChildNodes))
            {
               $check2 = $filename2->item(0)->getFirstChild->toString;
            }
            if ($childNode->getAttribute ('printCheck') ne '')
            {
               if (($childNode->getAttribute ('printCheck') == 1) ||
                   ($childNode->getAttribute ('printCheck') == 0))
               {
                  $printCheck = $childNode->getAttribute ('printCheck');
               }
            }
         }
         elsif ($nodeName eq 'checkScript')
         {
            $checkScript = $childNode->getFirstChild->toString;
            if ($childNode->getAttribute ('printCheck') ne '')
            {
               if (($childNode->getAttribute ('printCheck') == 1) ||
                   ($childNode->getAttribute ('printCheck') == 0))
               {
                  $printCheck = $childNode->getAttribute ('printCheck');
               }
            }
         }
         elsif (($nodeName eq 'trial') && ($inTrial == 0))
         {
            print "Trial $trialNumber:\n\n";

            my ($trialScore, $trialTotalScore) = handle_node (1, $childNode, $compile, $run, 
              $check1, $check2, $checkScript, $printCheck, $maximumCPU, $maximumMemory, $maximumFile);

            print "Trial points: $trialScore / $trialTotalScore\n\n";

            $score += $trialScore;
            $totalScore += $trialTotalScore;
            $trialNumber++;
         }
      }
   }

   if ($inTrial == 1)
   {
      if ($run =~ /(java|python)/)
      { 
         $nice = ''; 
      }
      print "Executing program\n\n";
      system "cd $ARGV[1]; $safe $maximumCPU $maximumMemory $maximumFile $nice $run";

      if ($checkScript ne '')
      {
         print "Invoking test framework\n\n";
      
         system "cd $ARGV[1]; rm -f trialscore.dat; $safe $maximumCPU $maximumMemory $maximumFile $checkScript";

         open (my $markFile, "$ARGV[1]/trialscore.dat");
         my $checkMark = <$markFile>;
         close ($markFile);
         
         my $trialmarks = 100;
         my $trialmarknode = $node->getElementsByTagName ('marks');
         if (($trialmarknode->getLength > 0) && 
             ($trialmarknode->item(0)->hasChildNodes) && 
             ($trialmarknode->item(0)->getFirstChild->toString =~ /[0-9]/))
         {
            $trialmarks = $trialmarknode->item(0)->getFirstChild->toString;
            $trialmarks =~ s/[^0-9]//o;
         }

         $totalScore = $trialmarks;
         if ($checkMark =~ /^[0-9]+$/)
         { $score = $checkMark; }
      }
      elsif (($check1 ne '') && ($check2 ne ''))
      {
         print "Comparing output\n\n";
         open (my $file1, "$ARGV[1]/$check1");
         my @lines1 = <$file1>;
         close ($file1);
         open (my $file2, "$ARGV[1]/$check2");
         my @lines2 = <$file2>;
         close ($file2);

         while (($#lines1 > -1) && ($lines1[$#lines1] =~ /^[\n\r\t\s]*$/)) 
         { pop @lines1; }
         while (($#lines1 > -1) && ($lines1[0] =~ /^[\n\r\t\s]*$/))
         { shift @lines1; }
         while (($#lines2 > -1) && ($lines2[$#lines2] =~ /^[\n\r\t\s]*$/)) 
         { pop @lines2; }
         while (($#lines2 > -1) && ($lines2[0] =~ /^[\n\r\t\s]*$/)) 
         { shift @lines2; }
         
         my $equal = 1;
         if ($#lines1 != $#lines2)
         { $equal = 0; }
         else
         {
            for ( my $i=0; $i<=$#lines1; $i++ )
            {
               $lines1[$i] =~ s/[\n\r\t\s]+$//go;
               $lines2[$i] =~ s/[\n\r\t\s]+$//go;
               if ($lines1[$i] ne $lines2[$i])
               { $equal = 0; }
            }
         }

         my $trialmarks = 100;
         my $trialmarknode = $node->getElementsByTagName ('marks');
         if (($trialmarknode->getLength > 0) && 
             ($trialmarknode->item(0)->hasChildNodes) && 
             ($trialmarknode->item(0)->getFirstChild->toString =~ /[0-9]/))
         {
            $trialmarks = $trialmarknode->item(0)->getFirstChild->toString;
            $trialmarks =~ s/[^0-9]//o;
         }

         my $penaltyMarks = 0;
         my $penaltyMarksNode = $node->getElementsByTagName ('penalty');
         if (($penaltyMarksNode->getLength > 0) && 
             ($penaltyMarksNode->item(0)->hasChildNodes) && 
             ($penaltyMarksNode->item(0)->getFirstChild->toString =~ /[0-9]/))
         {
            $penaltyMarks = $penaltyMarksNode->item(0)->getFirstChild->toString;
            $penaltyMarks =~ s/[^0-9]//o;
            $trialmarks = 0;
         }
         
         $totalScore = $trialmarks;

         if ($equal == 1)
         { 
            print "Output correct\n\n";
            $score = $trialmarks; 
         }
         else
         {
            print "Output not correct\n\n";
            $score = -$penaltyMarks;
            if ($printCheck == 1)
            {
               print "The expected output was:\n";
               foreach my $aline (@lines1) { chomp $aline; print $aline."\n"; }
               print "\n";
               print "Your program produced:\n";
               foreach my $aline (@lines2) { chomp $aline; print $aline."\n"; }
               print "\n";
               if (-e "$ARGV[1]/testinput.in")
               {
                  print "Input supplied to your program:\n";
                  open (my $inputfile, "$ARGV[1]/testinput.in");
                  while (my $inputline = <$inputfile>)
                  {
                     chomp $inputline;
                     print $inputline."\n";
                  }
                  close ($inputfile);
                  print "\n";
               }
            }
            print "<pre>Differences in the files are as follows:\n";
            open (my $difffile, "$diff $ARGV[1]/$check1 $ARGV[1]/$check2 |");
            while (my $diffline = <$difffile>)
            {
               print $diffline;
            }
            close ($difffile);
            print "</pre>\n";
         }
      }
   }

   ($score, $totalScore);
}

sub main
{
   print "Automarking ...\n\n";

   my $questionnumber = 1;

   print "Unzipping file\n";
   system "cd $ARGV[1]; $unzip -o \"$ARGV[0]\" ; $chmod -R a+rwx * ";

   my $score = 0;
   my $totalscore = 0;

   my $parser = new XML::DOM::Parser;
   my $markingguide = undef;

#open (my $f, ">t"); print $f $ARGV[2]; close ($f);
   if (-e $ARGV[2].'/markingguide.xml')
   {
      $markingguide = $parser->parsefile ($ARGV[2].'/markingguide.xml');
   }
   
   if (defined $markingguide)
   {
      foreach my $question ($markingguide->getElementsByTagName ('question'))
      {
         print "Question $questionnumber\n\n";
         
         my ($questionScore, $questionTotalScore) = handle_node (0, $question, '', '', '', '', '', 1, 
           $maximumCPUDefault, $maximumMemoryDefault, $maximumFileDefault);
         
         my $questionmarks = 100;
         my $questionmarknode = $question->getElementsByTagName ('marks');
         if (($questionmarknode->getLength > 0) && 
             ($questionmarknode->item(0)->hasChildNodes) &&
             ($questionmarknode->item(0)->getFirstChild->toString =~ /[0-9]/))
         {
            $questionmarks = $questionmarknode->item(0)->getFirstChild->toString;
            $questionmarks =~ s/[^0-9]//o;
         }
         $totalscore += $questionmarks;
         if ($questionTotalScore > 0)
         {
            $questionScore = $questionmarks * ($questionScore / $questionTotalScore);
            $score += $questionScore;
            print "Score for question: $questionScore\n\n";
         }
         
         $questionnumber++;
      }
   }   

   if ($score<0)
   { $score = 0; }

   if ($totalscore == 0)
   { $score = 0; }
   else
   { $score = $score * (100 / $totalscore); }
   my $truncscore = sprintf ("%.0f", $score);

   print "Total marks for assignment: $truncscore\n\n";

   unlink "$ARGV[1]/score.dat";
   open (my $scorefile, ">$ARGV[1]/score.dat");
   print $scorefile "$truncscore\n";
   close ($scorefile);
}

main;
